@@ -83,6 +83,9 @@ KSycocaEntry::Ptr KBuildSycoca::createEntry(const QString &file, bool addToFacto
     quint32 timeStamp = m_ctimeFactory->dict()->ctime(file, m_resource);
     if (!timeStamp) {
         timeStamp = calcResourceHash(m_resourceSubdir, file);
+        if (!timeStamp) { // file disappeared meanwhile
+            return {};
+        }
     }
     KSycocaEntry::Ptr entry;
     if (m_allEntries) {
@@ -148,10 +151,11 @@ bool KBuildSycoca::build()
     entryDictList.reserve(factories()->size());
     int i = 0;
     // For each factory
-    Q_FOREACH (KSycocaFactory* factory, *factories()) {
+    auto list = *factories();
+    for (KSycocaFactory* factory : qAsConst(list)) {
         KBSEntryDict *entryDict = new KBSEntryDict;
         if (m_allEntries) { // incremental build
-            Q_FOREACH (const KSycocaEntry::Ptr &entry, (*m_allEntries)[i++]) {
+            for (const KSycocaEntry::Ptr &entry : qAsConst((*m_allEntries)[i++])) {
                 //if (entry->entryPath().contains("fake"))
                 //    qCDebug(SYCOCA) << "inserting into entryDict:" << entry->entryPath() << entry;
                 entryDict->insert(entry->entryPath(), entry);
@@ -167,7 +171,8 @@ bool KBuildSycoca::build()
 
     // Save the mtime of each dir, just before we list them
     // ## should we convert to UTC to avoid surprises when summer time kicks in?
-    Q_FOREACH (const QString &dir, factoryResourceDirs()) {
+    const auto lstDirs = factoryResourceDirs();
+    for (const QString &dir : lstDirs) {
         qint64 stamp = 0;
         KSycocaUtilsPrivate::visitResourceDirectory(dir, [&stamp] (const QFileInfo &info) {
             stamp = qMax(stamp, info.lastModified().toMSecsSinceEpoch());
@@ -178,13 +183,14 @@ bool KBuildSycoca::build()
 
     QMap<QString, QByteArray> allResourcesSubDirs; // dirs, kstandarddirs-resource-name
     // For each factory
-    Q_FOREACH (KSycocaFactory* factory, *factories()) {
+    list = *factories();
+    for (KSycocaFactory* factory : qAsConst(list)) {
         // For each resource the factory deals with
         const KSycocaResourceList *list = factory->resourceList();
         if (!list) {
             continue;
         }
-        Q_FOREACH (const KSycocaResource &res, *list) {
+        for (const KSycocaResource &res : qAsConst(*list)) {
             // With this we would get dirs, but not a unique list of relative files (for global+local merging to work)
             //const QStringList dirs = QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, res.subdir, QStandardPaths::LocateDirectory);
             //allResourcesSubDirs[res.resource] += dirs;
@@ -203,7 +209,7 @@ bool KBuildSycoca::build()
         QSet<QString> relFiles;
         const QStringList dirs = QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, m_resourceSubdir, QStandardPaths::LocateDirectory);
         qCDebug(SYCOCA) << "Looking for subdir" << m_resourceSubdir << "=>" << dirs;
-        Q_FOREACH (const QString &dir, dirs) {
+        for (const QString &dir : dirs) {
             QDirIterator it(dir, QDirIterator::Subdirectories);
             while (it.hasNext()) {
                 const QString filePath = it.next();
@@ -228,7 +234,7 @@ bool KBuildSycoca::build()
                 continue;
             }
 
-            Q_FOREACH (const KSycocaResource &res, *list) {
+            for (const KSycocaResource &res : qAsConst(*list)) {
                 if (res.resource != (*it1)) {
                     continue;
                 }
@@ -272,7 +278,8 @@ bool KBuildSycoca::build()
 
         // Storing the mtime *after* looking at these dirs is a tiny race condition,
         // but I'm not sure how to get the vfolder dirs upfront...
-        Q_FOREACH (QString dir, m_vfolder->allDirectories()) {
+        const auto allDirectories = m_vfolder->allDirectories();
+        for (QString dir : allDirectories) {
             if (dir.endsWith(QLatin1Char('/'))) {
                 dir.chop(1); // remove trailing slash, to avoid having ~/.local/share/applications twice
             }
@@ -313,7 +320,7 @@ void KBuildSycoca::createMenu(const QString &caption_, const QString &name_, VFo
 {
     QString caption = caption_;
     QString name = name_;
-    foreach (VFolderMenu::SubMenu *subMenu, menu->subMenus) {
+    for (VFolderMenu::SubMenu *subMenu : qAsConst(menu->subMenus)) {
         QString subName = name + subMenu->name + QLatin1Char('/');
 
         QString directoryFile = subMenu->directoryFile;
@@ -355,7 +362,7 @@ void KBuildSycoca::createMenu(const QString &caption_, const QString &name_, VFo
     if (name.isEmpty()) {
         name += QLatin1Char('/');
     }
-    foreach (const KService::Ptr &p, menu->items) {
+    for (const KService::Ptr &p : qAsConst(menu->items)) {
         if (m_menuTest) {
             if (!menu->isDeleted && !p->noDisplay())
                 printf("%s\t%s\t%s\n", qPrintable(caption), qPrintable(p->menuId()),
@@ -449,8 +456,8 @@ bool KBuildSycoca::recreate(bool incremental)
         //as $HOME may also be that of another user rather than /root
 #ifdef Q_OS_UNIX
         if (qEnvironmentVariableIsSet("SUDO_UID")) {
-            const int uid = qgetenv("SUDO_UID").toInt();
-            const int gid = qgetenv("SUDO_GID").toInt();
+            const int uid = qEnvironmentVariableIntValue("SUDO_UID");
+            const int gid = qEnvironmentVariableIntValue("SUDO_GID");
             if (uid && gid) {
                 fchown(database.handle(), uid, gid);
             }
@@ -513,7 +520,8 @@ void KBuildSycoca::save(QDataStream *str)
     //KSycocaFactory * servicetypeFactory = 0;
     //KBuildMimeTypeFactory * mimeTypeFactory = 0;
     KBuildServiceFactory *serviceFactory = nullptr;
-    Q_FOREACH (KSycocaFactory* factory, *factories()) {
+    auto lst = *factories();
+    for (KSycocaFactory* factory : qAsConst(lst)) {
         qint32 aId;
         qint32 aOffset;
         aId = factory->factoryId();
@@ -547,7 +555,8 @@ void KBuildSycoca::save(QDataStream *str)
     qCDebug(SYCOCA) << "Saving";
 
     // Write factory data....
-    Q_FOREACH (KSycocaFactory* factory, *factories()) {
+    lst = *factories();
+    for (KSycocaFactory* factory : qAsConst(lst)) {
         factory->save(*str);
         if (str->status() != QDataStream::Ok) { // ######## TODO: does this detect write errors, e.g. disk full?
             return;    // error
@@ -560,7 +569,8 @@ void KBuildSycoca::save(QDataStream *str)
     str->device()->seek(0);
 
     (*str) << qint32(KSycoca::version());
-    Q_FOREACH (KSycocaFactory* factory, *factories()) {
+    lst = *factories();
+    for (KSycocaFactory* factory : qAsConst(lst)) {
         qint32 aId;
         qint32 aOffset;
         aId = factory->factoryId();
@@ -615,7 +625,7 @@ static quint32 updateHash(const QString &file, quint32 hash)
     if (fi.isReadable() && fi.isFile()) {
         // This was using buff.st_ctime (in Waldo's initial commit to kstandarddirs.cpp in 2001), but that looks wrong?
         // Surely we want to catch manual editing, while a chmod doesn't matter much?
-        hash += fi.lastModified().toTime_t();
+        hash += fi.lastModified().toSecsSinceEpoch();
     }
     return hash;
 }
@@ -627,14 +637,19 @@ quint32 KBuildSycoca::calcResourceHash(const QString &resourceSubDir, const QStr
         return updateHash(filename, hash);
     }
     const QStringList files = QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, resourceSubDir + QLatin1Char('/') + filename);
-    Q_FOREACH (const QString &file, files) {
+    for (const QString &file : files) {
         hash = updateHash(file, hash);
     }
     if (hash == 0 && !filename.endsWith(QLatin1String("update_ksycoca"))
             && !filename.endsWith(QLatin1String(".directory")) // bug? needs investigation from someone who understands the VFolder spec
        ) {
-        qCWarning(SYCOCA) << "File not found or not readable:" << filename << "found:" << files;
-        Q_ASSERT(hash != 0);
+        if (files.isEmpty()) {
+            // This can happen if the file was deleted between directory listing and the above locateAll
+            qCDebug(SYCOCA) << "File not found anymore:" << filename << " -- probably deleted meanwhile";
+        } else {
+            // This can happen if the file was deleted between locateAll and QFileInfo
+            qCDebug(SYCOCA) << "File(s) found but not readable (or disappeared meanwhile)" << files;
+        }
     }
     return hash;
 }
